@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,13 +14,13 @@
 
 from datetime import datetime
 from json import dumps, loads
+from typing import Any, Optional
 from urllib.parse import quote_plus
 
 from requests.status_codes import codes
 
-from rucio.client.baseclient import BaseClient
-from rucio.client.baseclient import choice
-from rucio.common.utils import build_url, render_json, chunks
+from rucio.client.baseclient import BaseClient, choice
+from rucio.common.utils import build_url, chunks, render_json
 
 
 class ReplicaClient(BaseClient):
@@ -144,8 +143,6 @@ class ReplicaClient(BaseClient):
         :param rse_expression: The RSE expression to restrict replicas on a set of RSEs.
         :param client_location: Client location dictionary for PFN modification {'ip', 'fqdn', 'site', 'latitude', 'longitude'}
         :param sort: Sort the replicas: ``geoip`` - based on src/dst IP topographical distance
-                                        ``closeness`` - based on src/dst closeness
-                                        ``dynamic`` - Rucio Dynamic Smart Sort (tm)
         :param domain: Define the domain. None is fallback to 'wan', otherwise 'wan, 'lan', or 'all'
         :param signature_lifetime: If supported, in seconds, restrict the lifetime of the signed PFN.
         :param nrandom: pick N random replicas. If the initial number of replicas is smaller than N, returns all replicas.
@@ -235,7 +232,17 @@ class ReplicaClient(BaseClient):
         exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
         raise exc_cls(exc_msg)
 
-    def add_replica(self, rse, scope, name, bytes_, adler32, pfn=None, md5=None, meta={}):
+    def add_replica(
+        self,
+        rse: str,
+        scope: str,
+        name: str,
+        bytes_: int,
+        adler32: str,
+        pfn: Optional[str] = None,
+        md5: Optional[str] = None,
+        meta: Optional[dict[str, Any]] = None
+    ) -> bool:
         """
         Add file replicas to a RSE.
 
@@ -251,6 +258,7 @@ class ReplicaClient(BaseClient):
         :return: True if files were created successfully.
 
         """
+        meta = meta or {}
         dict_ = {'scope': scope, 'name': name, 'bytes': bytes_, 'meta': meta, 'adler32': adler32}
         if md5:
             dict_['md5'] = md5
