@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,20 +17,25 @@
 import configparser
 import json
 import os
-from collections.abc import Callable
-from typing import TYPE_CHECKING, overload, Optional, TypeVar, Union
+from functools import cache
+from typing import TYPE_CHECKING, Any, Optional, TypeVar, Union, overload
 
 from rucio.common import exception
-from rucio.common.exception import ConfigNotFound, DatabaseException
+from rucio.common.exception import ConfigLoadingError, ConfigNotFound, DatabaseException
 
 _T = TypeVar('_T')
 _U = TypeVar('_U')
 
 if TYPE_CHECKING:
+    from collections.abc import Callable
+
     from sqlalchemy.orm import Session
 
+LEGACY_SECTION_NAME = {}
+LEGACY_OPTION_NAME = {}
 
-def convert_to_any_type(value) -> Union[bool, int, float, str]:
+
+def convert_to_any_type(value: str) -> Union[bool, int, float, str]:
     if value.lower() in ['true', 'yes', 'on']:
         return True
     elif value.lower() in ['false', 'no', 'off']:
@@ -46,7 +50,7 @@ def convert_to_any_type(value) -> Union[bool, int, float, str]:
     return value
 
 
-def _convert_to_boolean(value):
+def _convert_to_boolean(value: Union[str, bool]) -> bool:
     if isinstance(value, bool):
         return value
     if value.lower() in ['true', 'yes', 'on', '1']:
@@ -111,7 +115,7 @@ def config_get(
         session: "Optional[Session]" = ...,
         use_cache: bool = ...,
         expiration_time: int = ...,
-        convert_type_fnc: Callable[[str], _T],
+        convert_type_fnc: 'Callable[[str], _T]',
 ) -> _T:
     ...
 
@@ -127,7 +131,7 @@ def config_get(
         session: "Optional[Session]" = ...,
         use_cache: bool = ...,
         expiration_time: int = ...,
-        convert_type_fnc: Callable[[str], _U],
+        convert_type_fnc: 'Callable[[str], _U]',
 ) -> Union[_T, _U]:
     ...
 
@@ -144,7 +148,7 @@ def config_get(
         session: "Optional[Session]" = ...,
         use_cache: bool = ...,
         expiration_time: int = ...,
-        convert_type_fnc: Callable[[str], _U],
+        convert_type_fnc: 'Callable[[str], _U]',
 ) -> Union[_T, _U]:
     ...
 
@@ -159,7 +163,7 @@ def config_get(
         session: "Optional[Session]" = None,
         use_cache: bool = True,
         expiration_time: int = 900,
-        convert_type_fnc: Callable[[str], _T] = lambda x: x,
+        convert_type_fnc: 'Callable[[str], _T]' = lambda x: x,
 ) -> Union[_T, _U]:
     """
     Return the string value for a given option in a section
@@ -189,15 +193,15 @@ def config_get(
     """
     try:
         return convert_type_fnc(get_config().get(section, option))
-    except (configparser.NoOptionError, configparser.NoSectionError, RuntimeError) as err:
+    except (configparser.NoOptionError, configparser.NoSectionError, ConfigNotFound) as err:
         try:
             legacy_config = get_legacy_config(section, option)
             if legacy_config is not None:
                 return convert_type_fnc(legacy_config)
-        except RuntimeError:
+        except ConfigNotFound:
             pass
 
-        from rucio.common.utils import is_client
+        from rucio.common.client import is_client
         client_mode = is_client()
 
         if not client_mode and check_config_table:
@@ -224,8 +228,6 @@ def get_legacy_config(section: str, option: str):
     :param option: The option of the new config.
     :returns: The string value of the legacy option if one is found, None otherwise.
     """
-    LEGACY_SECTION_NAME = {}
-    LEGACY_OPTION_NAME = {}
 
     section = LEGACY_SECTION_NAME.get(section, section)
     option = LEGACY_OPTION_NAME.get(option, option)
@@ -298,7 +300,7 @@ def config_get_int(
 def config_get_int(
         section: str,
         option: str,
-        raise_exception,
+        raise_exception: bool,
         default: _T = ...,
         *,
         check_config_table: bool = ...,
@@ -310,14 +312,14 @@ def config_get_int(
 
 
 def config_get_int(
-        section,
-        option,
-        raise_exception=True,
+        section: str,
+        option: str,
+        raise_exception: bool = True,
         default=None,
-        check_config_table=True,
-        session=None,
-        use_cache=True,
-        expiration_time=900,
+        check_config_table: bool = True,
+        session: "Optional[Session]" = None,
+        use_cache: bool = True,
+        expiration_time: int = 900,
 ):
     """
     Return the integer value for a given option in a section
@@ -396,14 +398,14 @@ def config_get_float(
 
 
 def config_get_float(
-        section,
-        option,
-        raise_exception=True,
+        section: str,
+        option: str,
+        raise_exception: bool = True,
         default=None,
-        check_config_table=True,
-        session=None,
-        use_cache=True,
-        expiration_time=900,
+        check_config_table: bool = True,
+        session: "Optional[Session]" = None,
+        use_cache: bool = True,
+        expiration_time: int = 900,
 ):
     """
     Return the floating point value for a given option in a section
@@ -483,14 +485,14 @@ def config_get_bool(
 
 
 def config_get_bool(
-        section,
-        option,
-        raise_exception=True,
+        section: str,
+        option: str,
+        raise_exception: bool = True,
         default=None,
-        check_config_table=True,
-        session=None,
-        use_cache=True,
-        expiration_time=900,
+        check_config_table: bool = True,
+        session: "Optional[Session]" = None,
+        use_cache: bool = True,
+        expiration_time: int = 900,
 ):
     """
     Return the boolean value for a given option in a section
@@ -570,14 +572,14 @@ def config_get_list(
 
 
 def config_get_list(
-        section,
-        option,
-        raise_exception=True,
+        section: str,
+        option: str,
+        raise_exception: bool = True,
         default=None,
-        check_config_table=True,
-        session=None,
-        use_cache=True,
-        expiration_time=900,
+        check_config_table: bool = True,
+        session: "Optional[Session]" = None,
+        use_cache: bool = True,
+        expiration_time: int = 900,
 ):
     """
     Return a list for a given option in a section
@@ -638,7 +640,7 @@ def __config_get_table(
         session: "Optional[Session]" = None,
         use_cache: bool = True,
         expiration_time: int = 900,
-        convert_type_fnc: Optional[Callable[[str], _T]],
+        convert_type_fnc: Optional['Callable[[str], _T]'],
 ) -> _T:
     """
     Search for a section-option configuration parameter in the configuration table
@@ -656,7 +658,6 @@ def __config_get_table(
     :raises ConfigNotFound
     :raises DatabaseException
     """
-    global __CONFIG
     try:
         from rucio.core.config import get as core_config_get
         return core_config_get(section, option, default=default, session=session, use_cache=use_cache,
@@ -665,7 +666,7 @@ def __config_get_table(
         if raise_exception and default is None:
             raise err
         if clean_cached:
-            __CONFIG = None
+            clean_cached_config()
         return default
 
 
@@ -692,7 +693,7 @@ def config_remove_option(section: str, option: str) -> bool:
     return get_config().remove_option(section, option)
 
 
-def config_set(section: str, option: str, value: str):
+def config_set(section: str, option: str, value: str) -> None:
     """
     Set a configuration option in a given section.
 
@@ -705,37 +706,35 @@ def config_set(section: str, option: str, value: str):
     return get_config().set(section, option, value)
 
 
-def get_config_dirs():
+def get_config_dirs() -> list[str]:
     """
     Returns all available configuration directories in order:
     - $RUCIO_HOME/etc/
     - $VIRTUAL_ENV/etc/
+    - $CONDA_PREFIX/etc
     - /opt/rucio/
     """
     configdirs = []
 
-    if 'RUCIO_HOME' in os.environ:
-        configdirs.append('%s/etc/' % os.environ['RUCIO_HOME'])
-
-    if 'VIRTUAL_ENV' in os.environ:
-        configdirs.append('%s/etc/' % os.environ['VIRTUAL_ENV'])
+    env_vars = ['RUCIO_HOME', 'VIRTUAL_ENV', 'CONDA_PREFIX']
+    configdirs.extend([os.path.join(os.environ[var], 'etc', '') for var in env_vars if var in os.environ])
 
     configdirs.append('/opt/rucio/etc/')
 
     return configdirs
 
 
-def get_lfn2pfn_algorithm_default():
+def get_lfn2pfn_algorithm_default() -> str:
     """Returns the default algorithm name for LFN2PFN translation for this server."""
     default_lfn2pfn = "hash"
     try:
         default_lfn2pfn = config_get('policy', 'lfn2pfn_algorithm_default')
-    except (configparser.NoOptionError, configparser.NoSectionError, RuntimeError):
+    except (configparser.NoOptionError, configparser.NoSectionError, ConfigNotFound, RuntimeError):
         pass
     return default_lfn2pfn
 
 
-def get_rse_credentials(path_to_credentials_file: Optional[Union[str, os.PathLike]] = None):
+def get_rse_credentials(path_to_credentials_file: Optional[Union[str, os.PathLike]] = None) -> dict[str, Any]:
     """ Returns credentials for RSEs. """
 
     path = ''
@@ -755,21 +754,15 @@ def get_rse_credentials(path_to_credentials_file: Optional[Union[str, os.PathLik
     return credentials
 
 
-__CONFIG = None
-
-
+@cache
 def get_config() -> configparser.ConfigParser:
     """Factory function for the configuration class. Returns the ConfigParser instance."""
-    global __CONFIG
-    if __CONFIG is None:
-        __CONFIG = Config()
-    return __CONFIG.parser
+    return Config().parser
 
 
-def clean_cached_config():
+def clean_cached_config() -> None:
     """Deletes the cached config singleton instance."""
-    global __CONFIG
-    __CONFIG = None
+    get_config.cache_clear()
 
 
 class Config:
@@ -786,11 +779,10 @@ class Config:
             configs = [os.path.join(confdir, 'rucio.cfg') for confdir in get_config_dirs()]
             self.configfile = next(iter(filter(os.path.exists, configs)), None)
             if self.configfile is None:
-                raise RuntimeError('Could not load Rucio configuration file. '
-                                   'Rucio looked in the following paths for a configuration file, in order:'
-                                   '\n\t' + '\n\t'.join(configs))
+                raise ConfigNotFound(
+                    'Could not load Rucio configuration file. '
+                    'Rucio looked in the following paths for a configuration file, in order:'
+                    '\n\t' + '\n\t'.join(configs))
 
         if not self.parser.read(self.configfile) == [self.configfile]:
-            raise RuntimeError('Could not load Rucio configuration file. '
-                               'Rucio tried loading the following configuration file:'
-                               '\n\t' + self.configfile)
+            raise ConfigLoadingError(self.configfile)
