@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,12 +19,12 @@ import pytest
 from rucio.client.didclient import DIDClient
 from rucio.common.exception import KeyNotFound
 from rucio.common.utils import generate_uuid
-from rucio.core.did import add_did, delete_dids, set_metadata_bulk, set_dids_metadata_bulk
-from rucio.core.did_meta_plugins import list_dids, get_metadata, set_metadata
+from rucio.core.did import add_did, delete_dids, set_dids_metadata_bulk, set_metadata_bulk
+from rucio.core.did_meta_plugins import get_metadata, list_dids, set_metadata
 from rucio.core.did_meta_plugins.mongo_meta import MongoDidMeta
 from rucio.core.did_meta_plugins.postgres_meta import ExternalPostgresJSONDidMeta
 from rucio.db.sqla.util import json_implemented
-from rucio.tests.common import skip_rse_tests_with_accounts, did_name_generator
+from rucio.tests.common import did_name_generator, skip_rse_tests_with_accounts
 
 
 def skip_without_json():
@@ -193,14 +192,24 @@ class TestDidMetaJSON:
         assert [tmp_dsn4] == results
 
 
-@pytest.fixture
-def mongo_meta():
-    return MongoDidMeta(
-        host='mongo',
-        port=27017,
-        db='test_db',
-        collection='test_collection'
-    )
+@pytest.fixture(params=["with-auth", "no-auth"])
+def mongo_meta(request):
+    if request.param == "with-auth":
+        return MongoDidMeta(
+            host='mongo',
+            port=27017,
+            db='test_db',
+            collection='test_collection',
+            user="rucio",
+            password="mongo-meta",
+        )
+    else:
+        return MongoDidMeta(
+            host='mongo-noauth',
+            port=27017,
+            db='test_db',
+            collection='test_collection',
+        )
 
 
 @skip_rse_tests_with_accounts
