@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,41 +17,36 @@ import os
 import time
 from datetime import datetime, timedelta
 from json import dumps
+from typing import TYPE_CHECKING
 from unittest import mock
 from xml.etree import ElementTree
 
 import pytest
 import xmltodict
-from werkzeug.datastructures import MultiDict
-from werkzeug.datastructures import Headers
+from werkzeug.datastructures import Headers, MultiDict
 
 from rucio.client.ruleclient import RuleClient
-from rucio.common.exception import (DataIdentifierNotFound, AccessDenied, RucioException,
-                                    ReplicaIsLocked, ReplicaNotFound, ScopeNotFound,
-                                    DatabaseException, InputValidationError)
+from rucio.common.constants import RseAttr
+from rucio.common.exception import AccessDenied, DatabaseException, DataIdentifierNotFound, InputValidationError, ReplicaIsLocked, ReplicaNotFound, RucioException, ScopeNotFound
 from rucio.common.schema import get_schema_value
-from rucio.common.utils import generate_uuid, clean_surls, parse_response
+from rucio.common.utils import clean_pfns, generate_uuid, parse_response
 from rucio.core.config import set as cconfig_set
-from rucio.core.did import add_did, attach_dids, get_did, set_status, list_files, get_did_atime
-from rucio.core.replica import (add_replica, add_replicas, delete_replicas, get_replicas_state,
-                                get_replica, list_replicas, update_replica_state,
-                                get_RSEcoverage_of_dataset, get_replica_atime,
-                                touch_replica, get_bad_pfns, set_tombstone, add_bad_dids)
+from rucio.core.did import add_did, attach_dids, get_did, get_did_atime, list_files, set_status
+from rucio.core.replica import add_bad_dids, add_replica, add_replicas, delete_replicas, get_bad_pfns, get_replica, get_replica_atime, get_replicas_state, get_rse_coverage_of_dataset, list_replicas, set_tombstone, touch_replica, update_replica_state
 from rucio.core.rse import add_protocol, add_rse_attribute, del_rse_attribute
 from rucio.daemons.badreplicas.minos import minos
 from rucio.daemons.badreplicas.minos_temporary_expiration import minos_tu_expiration
 from rucio.db.sqla import models
-from rucio.db.sqla.constants import DIDType, ReplicaState, BadPFNStatus, OBSOLETE
+from rucio.db.sqla.constants import OBSOLETE, BadPFNStatus, DIDType, ReplicaState
 from rucio.db.sqla.session import transactional_session
 from rucio.rse import rsemanager as rsemgr
-from rucio.tests.common import execute, headers, auth, Mime, accept, did_name_generator
+from rucio.tests.common import Mime, accept, auth, did_name_generator, execute, headers
 
-from typing import TYPE_CHECKING
 if TYPE_CHECKING:
     from .temp_factories import TemporaryRSEFactory
 
 
-def mocked_VP_requests_get(*args, **kwargs):
+def mocked_vp_requests_get(*args, **kwargs):
     """This method will be used by the mock to replace requests.get to VP server."""
     class MockResponse:
         def __init__(self, json_data, status_code):
@@ -85,7 +79,7 @@ def mocked_VP_requests_get(*args, **kwargs):
 
 class TestReplicaCore:
 
-    @mock.patch('rucio.core.replica.requests.get', side_effect=mocked_VP_requests_get)
+    @mock.patch('rucio.core.replica.requests.get', side_effect=mocked_vp_requests_get)
     @pytest.mark.skipif(os.environ.get('POLICY') != 'atlas', reason='Hard-coded ATLAS PFN convention')
     def test_cache_replicas(self, mock_get, rse_factory, mock_scope, root_account):
         """ REPLICA (CORE): Test listing replicas with cached root protocol """
@@ -224,7 +218,7 @@ class TestReplicaCore:
 
         if simulate_multirange:
             add_rse_attribute(
-                rse_id=rse_id, key='simulate_multirange', value=str(nconns)
+                rse_id=rse_id, key=RseAttr.SIMULATE_MULTIRANGE, value=str(nconns)
             )
 
         # add files
@@ -463,7 +457,7 @@ class TestReplicaCore:
                                   'lan': {'read': 1, 'write': 1, 'delete': 1},
                                   'wan': {'read': 1, 'write': 1, 'delete': 1}}})
 
-        add_rse_attribute(rse_id=rse_id, key='site', value='APERTURE')
+        add_rse_attribute(rse_id=rse_id, key=RseAttr.SITE, value='APERTURE')
 
         files = [{'scope': mock_scope, 'name': 'element_%s' % generate_uuid(),
                   'bytes': 1234, 'adler32': 'deadbeef'}]
@@ -474,7 +468,7 @@ class TestReplicaCore:
         replicas = [r for r in replica_client.list_replicas(dids=[{'scope': 'mock', 'name': f['name']} for f in files],
                                                             client_location={'site': 'SOMEWHERE'})]
         assert 'root://' in list(replicas[0]['pfns'].keys())[0]
-        del_rse_attribute(rse_id=rse_id, key='site')
+        del_rse_attribute(rse_id=rse_id, key=RseAttr.SITE)
 
         replicas = [r for r in replica_client.list_replicas(dids=[{'scope': 'mock', 'name': f['name']} for f in files])]
         assert 'root://' in list(replicas[0]['pfns'].keys())[0]
@@ -550,7 +544,7 @@ class TestReplicaCore:
         rse2, rse2_id = rse_factory.make_mock_rse()
         activity = get_schema_value('ACTIVITY')['enum'][0]
         tombstone_delay = 3600
-        add_rse_attribute(rse_id=rse2_id, key='tombstone_delay', value=tombstone_delay)
+        add_rse_attribute(rse_id=rse2_id, key=RseAttr.TOMBSTONE_DELAY, value=tombstone_delay)
 
         # Will use the default tombstone delay
         did1 = did_factory.random_file_did()
@@ -622,7 +616,7 @@ class TestReplicaCore:
         for replica in list(list_replicas([{'scope': mock_scope, 'name': dsn}], all_states=True)):
             assert replica["states"][rse_id] == "BAD"
 
-    def test_get_RSE_coverage_of_dataset(self, rse_factory, mock_scope, root_account):
+    def test_get_rse_coverage_of_dataset(self, rse_factory, mock_scope, root_account):
         """ REPLICA (CORE): test RSE coverage retrieval """
         _, rse1_id = rse_factory.make_mock_rse()
         _, rse2_id = rse_factory.make_mock_rse()
@@ -632,7 +626,7 @@ class TestReplicaCore:
         add_did(scope=mock_scope, name=dsn, did_type='DATASET', account=root_account)
 
         # test empty dataset
-        cov = get_RSEcoverage_of_dataset(scope=mock_scope, name=dsn)
+        cov = get_rse_coverage_of_dataset(scope=mock_scope, name=dsn)
         print(cov)
         assert cov == {}
         # add files/replicas
@@ -644,7 +638,7 @@ class TestReplicaCore:
             add_replica(rse_id=rse3_id, scope=mock_scope, name=dsn + '_%06d.data' % i, bytes_=100, account=root_account)
 
         attach_dids(scope=mock_scope, name=dsn, dids=[{'scope': mock_scope, 'name': dsn + '_%06d.data' % i} for i in range(1, 16)], account=root_account)
-        cov = get_RSEcoverage_of_dataset(scope=mock_scope, name=dsn)
+        cov = get_rse_coverage_of_dataset(scope=mock_scope, name=dsn)
         print(cov)
         assert cov[rse1_id] == 700
         assert cov[rse2_id] == 300
@@ -917,7 +911,7 @@ def test_client_add_temporary_unavailable_pfns(rse_factory, mock_scope, replica_
         bad_pfns[res['pfn']] = (res['state'], res['reason'], res['expires_at'])
 
     for pfn in list_rep:
-        pfn = str(clean_surls([pfn])[0])
+        pfn = str(clean_pfns([pfn])[0])
         assert pfn in bad_pfns
         assert bad_pfns[pfn][0] == BadPFNStatus.TEMPORARY_UNAVAILABLE
         assert bad_pfns[pfn][1] == reason_str
@@ -947,7 +941,7 @@ def test_client_add_temporary_unavailable_pfns(rse_factory, mock_scope, replica_
         did['state'] = ReplicaState.TEMPORARY_UNAVAILABLE
         rep.append(did)
 
-    time.sleep(10)  # Test broken if minos_tu_expiration excuted immediately. Threading effect ?
+    time.sleep(10)  # Test broken if minos_tu_expiration executed immediately. Threading effect ?
     # Run the minos expiration
     minos_tu_expiration(once=True)
     # Check the state in the replica table
@@ -981,7 +975,7 @@ def test_client_declare_bad_pfns(rse_factory, mock_scope, replica_client):
         bad_pfns[res['pfn']] = (res['state'], res['reason'], res['expires_at'])
 
     for pfn in list_rep:
-        pfn = str(clean_surls([pfn])[0])
+        pfn = str(clean_pfns([pfn])[0])
         assert pfn in bad_pfns
         assert bad_pfns[pfn][0] == BadPFNStatus.BAD
         assert bad_pfns[pfn][1] == reason_str
@@ -1215,7 +1209,7 @@ def test_client_list_replicas_streaming_error(content_type, vo, did_client, repl
         assert metalink
         print(metalink)
         with pytest.raises(ElementTree.ParseError):
-            ElementTree.fromstring(metalink)
+            ElementTree.fromstring(metalink)  # noqa: S314
 
     elif content_type == Mime.JSON_STREAM:
         # for the json stream mimetype the API method just returns all mocked replicas on error
